//<c>   Copyright 1996-1998 by Gerry J. Danen; all rights reserved.

//<t>   Do something with older files

//      Syntax:
//          show:   OLDER days fspec [/D] [/NOSUB]
//          delete: OLDER days /DEL [/Q] fspec [/NOSUB]
//          move:   OLDER days /MOV [/Q] fspec newpath [/NOSUB]
//
//          /Q = quick, don't ask for confirmation
//          /D = show directories only

#include"gd_tools.h"
//#include"gd_extrn.h"
#include"gd_scrn.h"


#define pgVERS_ "V1.10"
#define pgTITL_ "Show and/or Process Aged Files"
#define pgCOPR_ "1996"


//#if     (! LARGE_MODEL)
//  #error LARGE MEMORY MODEL REQUIRED
//#endif


#define STR_SZ_ 133                             // string size


union   fileDateUnion                           // global unions
{
        ms_date_t dt ;
        unsigned pdt ;
}       fldate ;
union   fileTimeUnion
{
        ms_time_t tm ;
        unsigned ptm ;
}       fltime ;


int     finish_off_code = BAD_EXIT ;
ULONG   nDays = 0 ;
BOOL    slashDEL = FALSE, slashMOV = FALSE, slashD = FALSE, slashQ = FALSE,
        slashNOSUB = FALSE ;
UCHAR   sFspec[1024]="*.*", sNewpath[1024]="", chkSTRING[14] ;
char    origdir[FS_LEN] ;
INT32   sysYMD, sysNUMDAYS, chkYMD, chkNUMDAYS ;
INT16   sysYEAR, sysMTH, sysDAY ;
char    *months[] =
{
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};


void    main_exit( void )
{
    unsigned    numofdrives ;

    if ( finish_off_code == BAD_EXIT )
        fprintf( stderr, "\nAborting %s\n", get_global_program_name() );
    _dos_setdrive( origdir[0] - 64, &numofdrives );
    _chdir( origdir );                          // back to original
    fcloseall();
}


void    happy_ending( void )
{
    finish_off_code = GOOD_EXIT ;
    exit( GOOD_EXIT );
}


void    program_help( void )
{
    logon(1);
    fprintf( stderr,
            "This program is Year 2000 compliant. Certain assumptions are made if 2-digit\n"
            "years are used. See below for more information.\n\n"
            "Syntax:\n"
            "\tshow:   %s days|date fspec [/D] [/NOSUB]\n"
            "\tdelete: %s days|date /DEL fspec [/Q] [/NOSUB]\n"
            "\tmove:   %s days|date /MOV fspec newpath [/Q] [/NOSUB]\n"
            "\n"
            "\t/Q = quick, don't ask for confirmation\n"
            "\t/D = show directories only\n"
            "\t/NOSUB = do NOT process directories\n\n"
            "\tIf specifying date, use ddmmmyy, dd-mmm-yy, or dd-mmm-yyyy format.\n"
            "\t2-Digit years are converted to 4-digit years using %d as the century.\n"
            "\t(Century of today's date is used...)\n"
            "\n",
            get_global_program_name(),
            get_global_program_name(),
            get_global_program_name(),
            dt_todays_century4() );
    exit( BAD_EXIT );
}


void    do_show_parameters( void )              // show what we're going to do
{
    fprintf( stderr, "\n" );
    if ( slashDEL && slashMOV )
    {
        fprintf( stderr, "Cannot use /DEL and /MOV together\n" );
        exit( BAD_EXIT );
    }
    fprintf( stderr, "Today = %s, day number %ld\n", TodaysCDate(), sysNUMDAYS );
    fprintf( stderr, "Check = %s, day number %ld\n\n", chkSTRING, chkNUMDAYS );
    if ( slashDEL )
    {
        fprintf( stderr, "Deleting file(s) \"%s\", older than %ld day%s", sFspec, nDays, ONE_OR_MORE(nDays) );
        if ( slashQ )
            fprintf( stderr, " (quick mode)" );
        fprintf( stderr, "\n" );
    }
    else
    if ( slashMOV )
    {
        fprintf( stderr, "Moving file(s) \"%s\", older than %ld day%s to %s", sFspec, nDays, ONE_OR_MORE(nDays), sNewpath );
        if ( slashQ )
            fprintf( stderr, " (quick mode)" );
        fprintf( stderr, "\n" );
        if ( ! strlen(sNewpath) )
        {
            fprintf( stderr, "\nTarget path is missing...\n" );
            exit( BAD_EXIT );
        }
    }
    else
    {
        fprintf( stderr, "Showing file(s) \"%s\", older than %ld day%s", sFspec, nDays, ONE_OR_MORE(nDays) );
        if ( slashD )
            fprintf( stderr, " (directory names only)" );
        fprintf( stderr, "\n" );
    }
    if ( slashNOSUB )
        fprintf( stderr, "Not processing sub-directories\n" );
}


void    do_parameter( STRING p )
{
    static int  number=0 ;
    int         x ;
    INT32       ymd ;

    number ++ ;

    switch ( number )                           // 1=days; 2=fspec; 3=newpath
    {
        case 1:
            x = strlen(p);                      // ddmmmyy, dd-mmm-yy, or dd-mmm-yyyy format?
            if ( x == 7 )
                ymd = DateStr2ymd( p, DSTYP_DAT_DMY1 );
            else
            if ( x == 9 )
                ymd = DateStr2ymd( p, DSTYP_DAT_DMY2 );
            else
            if ( x == 11 )
                ymd = DateStr2ymd( p, DSTYP_DAT_DMY3 );
            else
                ymd = ERROR ;
            if ( ValidDate(ymd) == OK )
            {
                chkNUMDAYS = Date2Num( ymd );
                if ( chkNUMDAYS > sysNUMDAYS )
                {
                    printf( "Future date not allowed" );
                    exit( BAD_EXIT );
                }
                nDays = DaysBetween( sysYMD, ymd ); // dates in yyyymmdd format
            }
            else
                nDays = atol(p);
            chkYMD = NewDate( sysYMD, (INT16) (nDays * -1), 1 );
            chkNUMDAYS = Date2Num( chkYMD );
            FormatDate( chkYMD, chkSTRING, 9 );
//~~~ printf( "! nDays=%ld, chkYMD=%ld, chkNUMDAYS=%ld, chkSTRING=%s\n\n", nDays, chkYMD, chkNUMDAYS, chkSTRING );
            return ;
        case 2:
            ST_CPY( sFspec, p );
            return ;
        case 3:
            ST_CPY( sNewpath, p );
            return ;
    }
}


void    do_switch( STRING sw )
{
    if ( st_eq( sw, "DEL" ) )
        slashDEL = TRUE ;
    else
    if ( st_eq( sw, "MOV" ) )
        slashMOV = TRUE ;
    else
    if ( st_eq( sw, "D" ) )
        slashD = TRUE ;
    else
    if ( st_eq( sw, "Q" ) )
        slashQ = TRUE ;
    else
    if ( st_eq( sw, "NOSUB" ) )
        slashNOSUB = TRUE ;
}


void    main_init( int argc, char *argv[] )
{
    int i ;
    UCHAR       a[ STR_SZ_ ] ;

    tt_init( SHAREWARE_VERSION, 0, pgVERS_, pgTITL_, pgCOPR_ );
    logon(0);
    _getcwd( origdir, sizeof(origdir) );        // save current directory
    atexit( main_exit );
    dt_getsystemdate( &sysYMD, &sysYEAR, &sysMTH, &sysDAY );
    sysNUMDAYS = Date2Num( sysYMD );

    if ( argc < 3 )
        program_help();

    argc -= 1 ;
    LOOP( i, 1, argc, 1 )                       // process parameters
    {
        st_cpy( a, argv[i], STR_SZ_ );
        if ( a[0] == '/' )
            do_switch( &a[1] );
        else
            do_parameter( a );
    }
    do_show_parameters();
}


void    do_process_directory( void )            // recursive call
{
printf( "now at do_process_directory()\n" );
}


void    do_process( void )                      // go do it
{
//  Split fspec. If there's a path, that's where we start, otherwise
//  we start at the current directory.




    do_process_directory();

}


void    main( int argc, char *argv[] )
{
    main_init( argc, argv );
    do_process();
    happy_ending();
}
